@@ -17,6 +17,7 @@
  ***************************************************************************/
 
 #include "qgsdatasourceuri.h"
+#include "moc_qgsdatasourceuri.cpp"
 #include "qgsauthmanager.h"
 #include "qgslogger.h"
 #include "qgswkbtypes.h"
@@ -346,11 +347,26 @@ QString QgsDataSourceUri::database() const
   return mDatabase;
 }
 
+void QgsDataSourceUri::setPort( const QString &port )
+{
+  mPort = port;
+}
+
 QString QgsDataSourceUri::password() const
 {
   return mPassword;
 }
 
+void QgsDataSourceUri::setSslMode( SslMode mode )
+{
+  mSSLmode = mode;
+}
+
+void QgsDataSourceUri::setService( const QString &service )
+{
+  mService = service;
+}
+
 void QgsDataSourceUri::setPassword( const QString &password )
 {
   mPassword = password;
@@ -435,6 +451,11 @@ void QgsDataSourceUri::setSql( const QString &sql )
   mSql = sql;
 }
 
+void QgsDataSourceUri::setHost( const QString &host )
+{
+  mHost = host;
+}
+
 void QgsDataSourceUri::clearSchema()
 {
   mSchema.clear();
@@ -979,3 +1000,43 @@ QSet<QString> QgsDataSourceUri::parameterKeys() const
     paramKeys.insert( QLatin1String( "srid" ) );
   return paramKeys;
 }
+
+bool QgsDataSourceUri::operator==( const QgsDataSourceUri &other ) const
+{
+  // cheap comparisons first:
+  if ( mUseEstimatedMetadata != other.mUseEstimatedMetadata ||
+       mSelectAtIdDisabled != other.mSelectAtIdDisabled ||
+       mSelectAtIdDisabledSet != other.mSelectAtIdDisabledSet ||
+       mSSLmode != other.mSSLmode ||
+       mWkbType != other.mWkbType )
+  {
+    return false;
+  }
+
+  if ( mHost != other.mHost ||
+       mPort != other.mPort ||
+       mDriver != other.mDriver ||
+       mService != other.mService ||
+       mDatabase != other.mDatabase ||
+       mSchema != other.mSchema ||
+       mTable != other.mTable ||
+       mGeometryColumn != other.mGeometryColumn ||
+       mSql != other.mSql ||
+       mAuthConfigId != other.mAuthConfigId ||
+       mUsername != other.mUsername ||
+       mPassword != other.mPassword ||
+       mKeyColumn != other.mKeyColumn ||
+       mSrid != other.mSrid ||
+       mParams != other.mParams ||
+       mHttpHeaders != other.mHttpHeaders )
+  {
+    return false;
+  }
+
+  return true;
+}
+
+bool QgsDataSourceUri::operator!=( const QgsDataSourceUri &other ) const
+{
+  return !( *this == other );
+}
