@@ -43,9 +43,16 @@ QgsRenderContext::~QgsRenderContext() = default;
 QgsRenderContext::QgsRenderContext( const QgsRenderContext &rh )
   : QgsTemporalRangeObject( rh )
   , mFlags( rh.mFlags )
+  , mRasterizedRenderingPolicy( rh.mRasterizedRenderingPolicy )
   , mPainter( rh.mPainter )
   , mPreviewRenderPainter( rh.mPreviewRenderPainter )
   , mMaskPainter( rh.mMaskPainter )
+
+    // TODO -- these were NOT being copied, but it's unclear if that was intentional or a bug??
+  , mMaskIdProvider( nullptr )
+  , mCurrentMaskId( -1 )
+
+  , mIsGuiPreview( rh.mIsGuiPreview )
   , mCoordTransform( rh.mCoordTransform )
   , mDistanceArea( rh.mDistanceArea )
   , mExtent( rh.mExtent )
@@ -71,7 +78,7 @@ QgsRenderContext::QgsRenderContext( const QgsRenderContext &rh )
   , mTextRenderFormat( rh.mTextRenderFormat )
   , mRenderedFeatureHandlers( rh.mRenderedFeatureHandlers )
   , mHasRenderedFeatureHandlers( rh.mHasRenderedFeatureHandlers )
-  , mCustomRenderingFlags( rh.mCustomRenderingFlags )
+  , mCustomProperties( rh.mCustomProperties )
   , mDisabledSymbolLayers()
   , mClippingRegions( rh.mClippingRegions )
   , mFeatureClipGeometry( rh.mFeatureClipGeometry )
@@ -95,9 +102,14 @@ QgsRenderContext::QgsRenderContext( const QgsRenderContext &rh )
 QgsRenderContext &QgsRenderContext::operator=( const QgsRenderContext &rh )
 {
   mFlags = rh.mFlags;
+  mRasterizedRenderingPolicy = rh.mRasterizedRenderingPolicy;
   mPainter = rh.mPainter;
   mPreviewRenderPainter = rh.mPreviewRenderPainter;
   mMaskPainter = rh.mMaskPainter;
+  // TODO -- these were NOT being copied, but it's unclear if that was intentional or a bug??
+  // mMaskIdProvider
+  // mCurrentMaskId
+  mIsGuiPreview = rh.mIsGuiPreview;
   mCoordTransform = rh.mCoordTransform;
   mExtent = rh.mExtent;
   mOriginalMapExtent = rh.mOriginalMapExtent;
@@ -123,7 +135,7 @@ QgsRenderContext &QgsRenderContext::operator=( const QgsRenderContext &rh )
   mTextRenderFormat = rh.mTextRenderFormat;
   mRenderedFeatureHandlers = rh.mRenderedFeatureHandlers;
   mHasRenderedFeatureHandlers = rh.mHasRenderedFeatureHandlers;
-  mCustomRenderingFlags = rh.mCustomRenderingFlags;
+  mCustomProperties = rh.mCustomProperties;
   mClippingRegions = rh.mClippingRegions;
   mFeatureClipGeometry = rh.mFeatureClipGeometry;
   mTextureOrigin = rh.mTextureOrigin;
@@ -213,6 +225,7 @@ QgsFeedback *QgsRenderContext::feedback() const
 void QgsRenderContext::setFlags( Qgis::RenderContextFlags flags )
 {
   mFlags = flags;
+  matchRasterizedRenderingPolicyToFlags();
 }
 
 void QgsRenderContext::setFlag( Qgis::RenderContextFlag flag, bool on )
@@ -221,6 +234,7 @@ void QgsRenderContext::setFlag( Qgis::RenderContextFlag flag, bool on )
     mFlags |= flag;
   else
     mFlags &= ~( static_cast< int >( flag ) );
+  matchRasterizedRenderingPolicyToFlags();
 }
 
 Qgis::RenderContextFlags QgsRenderContext::flags() const
@@ -260,6 +274,9 @@ QgsRenderContext QgsRenderContext::fromMapSettings( const QgsMapSettings &mapSet
   ctx.setFlag( Qgis::RenderContextFlag::SkipSymbolRendering, mapSettings.testFlag( Qgis::MapSettingsFlag::SkipSymbolRendering ) );
   ctx.setFlag( Qgis::RenderContextFlag::RecordProfile, mapSettings.testFlag( Qgis::MapSettingsFlag::RecordProfile ) );
   ctx.setFlag( Qgis::RenderContextFlag::AlwaysUseGlobalMasks, mapSettings.testFlag( Qgis::MapSettingsFlag::AlwaysUseGlobalMasks ) );
+
+  ctx.setRasterizedRenderingPolicy( mapSettings.rasterizedRenderingPolicy() );
+
   ctx.setScaleFactor( mapSettings.outputDpi() / 25.4 ); // = pixels per mm
   ctx.setDpiTarget( mapSettings.dpiTarget() >= 0.0 ? mapSettings.dpiTarget() : -1.0 );
   ctx.setRendererScale( mapSettings.scale() );
@@ -277,7 +294,7 @@ QgsRenderContext QgsRenderContext::fromMapSettings( const QgsMapSettings &mapSet
   //this flag is only for stopping during the current rendering progress,
   //so must be false at every new render operation
   ctx.setRenderingStopped( false );
-  ctx.mCustomRenderingFlags = mapSettings.customRenderingFlags();
+  ctx.mCustomProperties = mapSettings.customRenderingFlags();
   ctx.setIsTemporal( mapSettings.isTemporal() );
   if ( ctx.isTemporal() )
     ctx.setTemporalRange( mapSettings.temporalRange() );
@@ -295,22 +312,35 @@ QgsRenderContext QgsRenderContext::fromMapSettings( const QgsMapSettings &mapSet
   ctx.mFrameRate = mapSettings.frameRate();
   ctx.mCurrentFrame = mapSettings.currentFrame();
 
+  const QStringList layerIds = mapSettings.layerIds( true );
+  if ( !layerIds.empty() )
+    ctx.setCustomProperty( QStringLiteral( "visible_layer_ids" ), layerIds );
+
   return ctx;
 }
 
 bool QgsRenderContext::forceVectorOutput() const
 {
-  return mFlags.testFlag( Qgis::RenderContextFlag::ForceVectorOutput );
+  return mRasterizedRenderingPolicy != Qgis::RasterizedRenderingPolicy::Default;
 }
 
 bool QgsRenderContext::useAdvancedEffects() const
 {
-  return mFlags.testFlag( Qgis::RenderContextFlag::UseAdvancedEffects );
+  return mRasterizedRenderingPolicy != Qgis::RasterizedRenderingPolicy::ForceVector;
 }
 
 void QgsRenderContext::setUseAdvancedEffects( bool enabled )
 {
   setFlag( Qgis::RenderContextFlag::UseAdvancedEffects, enabled );
+
+  if ( enabled && mRasterizedRenderingPolicy == Qgis::RasterizedRenderingPolicy::ForceVector )
+  {
+    mRasterizedRenderingPolicy = Qgis::RasterizedRenderingPolicy::PreferVector;
+  }
+  else if ( !enabled )
+  {
+    mRasterizedRenderingPolicy = Qgis::RasterizedRenderingPolicy::ForceVector;
+  }
 }
 
 bool QgsRenderContext::drawEditingInformation() const
@@ -336,6 +366,23 @@ void QgsRenderContext::setDrawEditingInformation( bool b )
 void QgsRenderContext::setForceVectorOutput( bool force )
 {
   setFlag( Qgis::RenderContextFlag::ForceVectorOutput, force );
+  if ( force && mRasterizedRenderingPolicy == Qgis::RasterizedRenderingPolicy::Default )
+  {
+    mRasterizedRenderingPolicy = Qgis::RasterizedRenderingPolicy::PreferVector;
+  }
+  else if ( !force )
+  {
+    switch ( mRasterizedRenderingPolicy )
+    {
+      case Qgis::RasterizedRenderingPolicy::Default:
+        break;
+
+      case Qgis::RasterizedRenderingPolicy::PreferVector:
+      case Qgis::RasterizedRenderingPolicy::ForceVector:
+        mRasterizedRenderingPolicy = Qgis::RasterizedRenderingPolicy::Default;
+        break;
+    }
+  }
 }
 
 void QgsRenderContext::setShowSelection( const bool showSelection )
@@ -452,7 +499,7 @@ double QgsRenderContext::convertToPainterUnits( double size, Qgis::RenderUnit un
     convertedSize *= symbologyReferenceScaleFactor;
   }
 
-  if ( mFlags & Qgis::RenderContextFlag::RenderSymbolPreview )
+  if ( mFlags & Qgis::RenderContextFlag::RenderSymbolPreview || mFlags & Qgis::RenderContextFlag::RenderLayerTree )
   {
     // apply property based constraints in order to optimise symbol preview rendering
     switch ( property )
@@ -478,6 +525,68 @@ double QgsRenderContext::convertToPainterUnits( double size, Qgis::RenderUnit un
   return convertedSize;
 }
 
+double QgsRenderContext::convertFromPainterUnits( double size, Qgis::RenderUnit unit ) const
+{
+  double conversionFactor = 1.0;
+  // NOLINTBEGIN(bugprone-branch-clone)
+  switch ( unit )
+  {
+    case Qgis::RenderUnit::Millimeters:
+      conversionFactor = 1 / mScaleFactor;
+      break;
+
+    case Qgis::RenderUnit::Points:
+      conversionFactor = POINTS_TO_MM / mScaleFactor;
+      break;
+
+    case Qgis::RenderUnit::Inches:
+      conversionFactor = 1 / ( mScaleFactor * INCH_TO_MM );
+      break;
+
+    case Qgis::RenderUnit::MetersInMapUnits:
+    {
+      if ( mMapToPixel.isValid() )
+        size = 1 / convertMetersToMapUnits( size );
+      // Fall through to RenderMapUnits with size in meters converted to size in MapUnits
+      [[fallthrough]];
+    }
+    case Qgis::RenderUnit::MapUnits:
+    {
+      if ( mMapToPixel.isValid() )
+      {
+        const double mup = mapToPixel().mapUnitsPerPixel();;
+        if ( mup > 0 )
+        {
+          conversionFactor = mup / 1.0;
+        }
+        else
+        {
+          conversionFactor = 1.0;
+        }
+      }
+      else
+      {
+        // invalid map to pixel. A size in map units can't be calculated, so treat the size as points.
+        // It's the best we can do in this situation!
+        conversionFactor = POINTS_TO_MM / mScaleFactor;
+      }
+      break;
+    }
+    case Qgis::RenderUnit::Pixels:
+      conversionFactor = 1.0;
+      break;
+
+    case Qgis::RenderUnit::Unknown:
+    case Qgis::RenderUnit::Percentage:
+      //no sensible value
+      conversionFactor = 1.0;
+      break;
+  }
+  // NOLINTEND(bugprone-branch-clone)
+
+  return size * conversionFactor;
+}
+
 double QgsRenderContext::convertToMapUnits( double size, Qgis::RenderUnit unit, const QgsMapUnitScale &scale ) const
 {
   const double mup = mMapToPixel.mapUnitsPerPixel();
@@ -628,6 +737,45 @@ double QgsRenderContext::convertMetersToMapUnits( double meters ) const
     case Qgis::DistanceUnit::Millimeters:
     case Qgis::DistanceUnit::Inches:
     case Qgis::DistanceUnit::Unknown:
+    case Qgis::DistanceUnit::ChainsInternational:
+    case Qgis::DistanceUnit::ChainsBritishBenoit1895A:
+    case Qgis::DistanceUnit::ChainsBritishBenoit1895B:
+    case Qgis::DistanceUnit::ChainsBritishSears1922Truncated:
+    case Qgis::DistanceUnit::ChainsBritishSears1922:
+    case Qgis::DistanceUnit::ChainsClarkes:
+    case Qgis::DistanceUnit::ChainsUSSurvey:
+    case Qgis::DistanceUnit::FeetBritish1865:
+    case Qgis::DistanceUnit::FeetBritish1936:
+    case Qgis::DistanceUnit::FeetBritishBenoit1895A:
+    case Qgis::DistanceUnit::FeetBritishBenoit1895B:
+    case Qgis::DistanceUnit::FeetBritishSears1922Truncated:
+    case Qgis::DistanceUnit::FeetBritishSears1922:
+    case Qgis::DistanceUnit::FeetClarkes:
+    case Qgis::DistanceUnit::FeetGoldCoast:
+    case Qgis::DistanceUnit::FeetIndian:
+    case Qgis::DistanceUnit::FeetIndian1937:
+    case Qgis::DistanceUnit::FeetIndian1962:
+    case Qgis::DistanceUnit::FeetIndian1975:
+    case Qgis::DistanceUnit::FeetUSSurvey:
+    case Qgis::DistanceUnit::LinksInternational:
+    case Qgis::DistanceUnit::LinksBritishBenoit1895A:
+    case Qgis::DistanceUnit::LinksBritishBenoit1895B:
+    case Qgis::DistanceUnit::LinksBritishSears1922Truncated:
+    case Qgis::DistanceUnit::LinksBritishSears1922:
+    case Qgis::DistanceUnit::LinksClarkes:
+    case Qgis::DistanceUnit::LinksUSSurvey:
+    case Qgis::DistanceUnit::YardsBritishBenoit1895A:
+    case Qgis::DistanceUnit::YardsBritishBenoit1895B:
+    case Qgis::DistanceUnit::YardsBritishSears1922Truncated:
+    case Qgis::DistanceUnit::YardsBritishSears1922:
+    case Qgis::DistanceUnit::YardsClarkes:
+    case Qgis::DistanceUnit::YardsIndian:
+    case Qgis::DistanceUnit::YardsIndian1937:
+    case Qgis::DistanceUnit::YardsIndian1962:
+    case Qgis::DistanceUnit::YardsIndian1975:
+    case Qgis::DistanceUnit::MilesUSSurvey:
+    case Qgis::DistanceUnit::Fathoms:
+    case Qgis::DistanceUnit::MetersGermanLegal:
       return ( meters * QgsUnitTypes::fromUnitToUnitFactor( Qgis::DistanceUnit::Meters, mDistanceArea.sourceCrs().mapUnits() ) );
   }
   return meters;
@@ -703,6 +851,31 @@ QSize QgsRenderContext::deviceOutputSize() const
   return outputSize() * mDevicePixelRatio;
 }
 
+Qgis::RasterizedRenderingPolicy QgsRenderContext::rasterizedRenderingPolicy() const
+{
+  return mRasterizedRenderingPolicy;
+}
+
+void QgsRenderContext::setRasterizedRenderingPolicy( Qgis::RasterizedRenderingPolicy policy )
+{
+  mRasterizedRenderingPolicy = policy;
+  switch ( mRasterizedRenderingPolicy )
+  {
+    case Qgis::RasterizedRenderingPolicy::Default:
+      mFlags.setFlag( Qgis::RenderContextFlag::ForceVectorOutput, false );
+      mFlags.setFlag( Qgis::RenderContextFlag::UseAdvancedEffects, true );
+      break;
+    case Qgis::RasterizedRenderingPolicy::PreferVector:
+      mFlags.setFlag( Qgis::RenderContextFlag::ForceVectorOutput, true );
+      mFlags.setFlag( Qgis::RenderContextFlag::UseAdvancedEffects, true );
+      break;
+    case Qgis::RasterizedRenderingPolicy::ForceVector:
+      mFlags.setFlag( Qgis::RenderContextFlag::ForceVectorOutput, true );
+      mFlags.setFlag( Qgis::RenderContextFlag::UseAdvancedEffects, false );
+      break;
+  }
+}
+
 double QgsRenderContext::frameRate() const
 {
   return mFrameRate;
@@ -733,6 +906,19 @@ void QgsRenderContext::setElevationMap( QgsElevationMap *map )
   mElevationMap = map;
 }
 
+void QgsRenderContext::matchRasterizedRenderingPolicyToFlags()
+{
+  if ( !mFlags.testFlag( Qgis::RenderContextFlag::ForceVectorOutput )
+       && mFlags.testFlag( Qgis::RenderContextFlag::UseAdvancedEffects ) )
+    mRasterizedRenderingPolicy = Qgis::RasterizedRenderingPolicy::Default;
+  else if ( mFlags.testFlag( Qgis::RenderContextFlag::ForceVectorOutput )
+            && mFlags.testFlag( Qgis::RenderContextFlag::UseAdvancedEffects ) )
+    mRasterizedRenderingPolicy = Qgis::RasterizedRenderingPolicy::PreferVector;
+  else if ( mFlags.testFlag( Qgis::RenderContextFlag::ForceVectorOutput )
+            && !mFlags.testFlag( Qgis::RenderContextFlag::UseAdvancedEffects ) )
+    mRasterizedRenderingPolicy = Qgis::RasterizedRenderingPolicy::ForceVector;
+}
+
 void QgsRenderContext::addSymbolLayerClipPath( const QString &symbolLayerId, QPainterPath path )
 {
   const QgsGeometry geometry = QgsGeometryPaintDevice::painterPathToGeometry( path );
