@@ -21,7 +21,7 @@
 #include "qgsstacitem.h"
 #include "qgsstacitemcollection.h"
 #include "qgsstaccollection.h"
-#include "qgsstaccollections.h"
+#include "qgsstaccollectionlist.h"
 
 
 constexpr int MAX_DISPLAYED_ITEMS = 20;
@@ -69,9 +69,7 @@ QVector<QgsDataItem *> QgsStacItemItem::createChildren()
 {
   QgsStacController *controller = stacController();
   QString error;
-  QgsStacObject *obj = controller->fetchStacObject( mPath, &error );
-  QgsStacItem *item = dynamic_cast<QgsStacItem *>( obj );
-  setStacItem( item );
+  setStacItem( controller->fetchStacObject<QgsStacItem>( mPath, &error ) );
 
   if ( !mStacItem )
     return { new QgsErrorItem( this, error, path() + QStringLiteral( "/error" ) ) };
@@ -87,10 +85,7 @@ bool QgsStacItemItem::hasDragEnabled() const
   const QMap<QString, QgsStacAsset> assets = mStacItem->assets();
   for ( auto it = assets.constBegin(); it != assets.constEnd(); ++it )
   {
-    if ( it->mediaType() == QLatin1String( "image/tiff; application=geotiff; profile=cloud-optimized" ) ||
-         it->mediaType() == QLatin1String( "image/vnd.stac.geotiff; cloud-optimized=true" ) ||
-         it->mediaType() == QLatin1String( "application/vnd.laszip+copc" ) ||
-         it->href().endsWith( QStringLiteral( "/ept.json" ) ) )
+    if ( it->isCloudOptimized() )
       return true;
   }
   return false;
@@ -137,7 +132,7 @@ QgsMimeDataUtils::UriList QgsStacItemItem::mimeUris() const
       uri.providerKey = QStringLiteral( "copc" );
       uri.uri = it->href();
     }
-    else if ( it->href().endsWith( QStringLiteral( "/ept.json" ) ) )
+    else if ( it->href().endsWith( QLatin1String( "/ept.json" ) ) )
     {
       uri.layerType = QStringLiteral( "pointcloud" );
       uri.providerKey = QStringLiteral( "ept" );
@@ -147,7 +142,7 @@ QgsMimeDataUtils::UriList QgsStacItemItem::mimeUris() const
     uris.append( uri );
   }
 
-  return { uris };
+  return uris;
 }
 
 bool QgsStacItemItem::equal( const QgsDataItem * )
@@ -178,9 +173,9 @@ QgsStacController *QgsStacItemItem::stacController()
   return nullptr;
 }
 
-void QgsStacItemItem::setStacItem( QgsStacItem *item )
+void QgsStacItemItem::setStacItem( std::unique_ptr<QgsStacItem> item )
 {
-  mStacItem.reset( item );
+  mStacItem = std::move( item );
   updateToolTip();
 }
 
@@ -192,15 +187,14 @@ QgsStacItem *QgsStacItemItem::stacItem() const
 void QgsStacItemItem::itemRequestFinished( int requestId, QString error )
 {
   QgsStacController *controller = stacController();
-  QgsStacObject *object = controller->takeStacObject( requestId );
-  QgsStacItem *item = dynamic_cast< QgsStacItem * >( object );
-  setStacItem( item );
-  if ( item )
+  std::unique_ptr< QgsStacItem > object = controller->takeStacObject< QgsStacItem >( requestId );
+  setStacItem( std::move( object ) );
+  if ( mStacItem )
   {
     mIconName = QStringLiteral( "mActionPropertiesWidget.svg" );
-    QString name = item->properties().value( QStringLiteral( "title" ), QString() ).toString();
+    QString name = mStacItem->properties().value( QStringLiteral( "title" ), QString() ).toString();
     if ( name.isEmpty() )
-      name = item->id();
+      name = mStacItem->id();
     mName = name;
   }
   else
@@ -300,7 +294,7 @@ void QgsStacCatalogItem::childrenCreated()
 
 void QgsStacCatalogItem::onControllerFinished( int requestId, const QString &error )
 {
-  for ( auto child : std::as_const( mChildren ) )
+  for ( QgsDataItem *child : std::as_const( mChildren ) )
   {
     if ( child->state() != Qgis::BrowserItemState::NotPopulated )
       continue;
@@ -317,44 +311,46 @@ void QgsStacCatalogItem::onControllerFinished( int requestId, const QString &err
 
 QVector<QgsDataItem *> QgsStacCatalogItem::createChildren()
 {
-  QgsStacCatalog *root = rootCatalog();
-  const bool supportsApi = root ? root->supportsStacApi() : false;
 
   QgsStacController *controller = stacController();
   QString error;
-  QgsStacObject *obj = controller->fetchStacObject( mPath, &error );
-  QgsStacCatalog *cat = dynamic_cast<QgsStacCatalog *>( obj );
-  setStacCatalog( cat );
+  setStacCatalog( controller->fetchStacObject< QgsStacCatalog >( mPath, &error ) );
 
   if ( !mStacCatalog )
     return { new QgsErrorItem( this, error, path() + QStringLiteral( "/error" ) ) };
 
+  QgsStacCatalog *root = rootCatalog();
+
+  const bool supportsCollections = root && root->conformsTo( QStringLiteral( "https://api.stacspec.org/v1.0.0/collections" ) );
+  const bool supportsItems = root && root->conformsTo( QStringLiteral( "https://api.stacspec.org/v1.0.0/ogcapi-features" ) );
+
   int itemsCount = 0;
   QVector<QgsDataItem *> contents;
   const QVector< QgsStacLink > links = mStacCatalog->links();
 
   // treat catalog/collection as static if it does not have a /items endpoint
-  bool hasItemsEndpoint = false;
-  bool hasCollectionsEndpoint = false;
-  if ( supportsApi )
+  bool useItemsEndpoint = false;
+  bool useCollectionsEndpoint = false;
+  if ( supportsCollections || supportsItems )
   {
-    for ( const auto &link : links )
+    for ( const QgsStacLink &link : links )
     {
       if ( link.relation() == QLatin1String( "items" ) )
       {
-        hasItemsEndpoint = true;
+        useItemsEndpoint = true;
       }
       else if ( link.relation() == QLatin1String( "data" ) &&
                 link.href().endsWith( QLatin1String( "/collections" ) ) )
       {
-        hasCollectionsEndpoint = true;
+        useCollectionsEndpoint = true;
       }
-      if ( hasItemsEndpoint && hasCollectionsEndpoint )
+      // if we found what we need we can stop looking
+      if ( supportsItems == useItemsEndpoint && supportsCollections == useCollectionsEndpoint )
         break;
     }
   }
 
-  for ( const auto &link : links )
+  for ( const QgsStacLink &link : links )
   {
     // skip hierarchical navigation links
     if ( link.relation() == QLatin1String( "self" ) ||
@@ -364,7 +360,7 @@ QVector<QgsDataItem *> QgsStacCatalogItem::createChildren()
       continue;
 
     if ( link.relation() == QLatin1String( "child" ) &&
-         !hasCollectionsEndpoint )
+         !useCollectionsEndpoint )
     {
       // may be either catalog or collection
       QgsStacCatalogItem *c = new QgsStacCatalogItem( this, link.title(), link.href() );
@@ -375,7 +371,7 @@ QVector<QgsDataItem *> QgsStacCatalogItem::createChildren()
     {
       // use /collections api
       QString error;
-      std::unique_ptr< QgsStacCollections > cols( controller->fetchCollections( link.href(), &error ) );
+      std::unique_ptr< QgsStacCollectionList > cols( controller->fetchCollections( link.href(), &error ) );
       if ( cols )
       {
         contents.append( createCollections( cols->takeCollections() ) );
@@ -388,7 +384,7 @@ QVector<QgsDataItem *> QgsStacCatalogItem::createChildren()
       }
     }
     else if ( link.relation() == QLatin1String( "item" ) &&
-              !hasItemsEndpoint )
+              !useItemsEndpoint )
     {
       itemsCount++;
 
@@ -398,7 +394,8 @@ QVector<QgsDataItem *> QgsStacCatalogItem::createChildren()
       QgsStacItemItem *i = new QgsStacItemItem( this, link.title(), link.href() );
       contents.append( i );
     }
-    else if ( link.relation() == QLatin1String( "items" ) )
+    else if ( link.relation() == QLatin1String( "items" ) &&
+              useItemsEndpoint )
     {
       // stac api items (ogcapi features)
       QString error;
@@ -456,15 +453,15 @@ void QgsStacCatalogItem::updateToolTip()
   }
 }
 
-void QgsStacCatalogItem::setStacCatalog( QgsStacCatalog *catalog )
+void QgsStacCatalogItem::setStacCatalog( std::unique_ptr<QgsStacCatalog> catalog )
 {
-  mStacCatalog.reset( catalog );
+  mStacCatalog = std::move( catalog );
   if ( mStacCatalog )
   {
     if ( mName.isEmpty() && !mStacCatalog->title().isEmpty() )
       setName( mStacCatalog->title() );
 
-    if ( mStacCatalog->type() == QgsStacObject::Collection )
+    if ( mStacCatalog->type() == Qgis::StacObjectType::Collection )
     {
       mIsCollection = true;
       mIconName = QStringLiteral( "mIconFolderOpen.svg" );
@@ -487,10 +484,12 @@ QVector< QgsDataItem * > QgsStacCatalogItem::createItems( const QVector<QgsStacI
     if ( !item )
       continue;
 
+    std::unique_ptr< QgsStacItem > object( item );
+
     const QString name = item->properties().value( QStringLiteral( "title" ), item->id() ).toString();
 
     QgsStacItemItem *i = new QgsStacItemItem( this, name, item->url() );
-    i->setStacItem( item );
+    i->setStacItem( std::move( object ) );
     i->setState( Qgis::BrowserItemState::Populated );
     contents.append( i );
   }
@@ -506,10 +505,12 @@ QVector<QgsDataItem *> QgsStacCatalogItem::createCollections( const QVector<QgsS
     if ( !col )
       continue;
 
+    std::unique_ptr< QgsStacCollection > object( col );
+
     const QString name = col->title().isEmpty() ? col->id() : col->title();
 
     QgsStacCatalogItem *i = new QgsStacCatalogItem( this, name, col->url() );
-    i->setStacCatalog( col );
+    i->setStacCatalog( std::move( object ) );
     contents.append( i );
   }
   return contents;
@@ -589,7 +590,7 @@ QgsStacRootItem::QgsStacRootItem( QgsDataItem *parent, const QString &name, cons
 QVector<QgsDataItem *> QgsStacRootItem::createChildren()
 {
   QVector<QgsDataItem *> connections;
-  const auto connectionList = QgsStacConnection::connectionList();
+  const QStringList connectionList = QgsStacConnection::connectionList();
   for ( const QString &connName : connectionList )
   {
     QgsDataItem *conn = new QgsStacConnectionItem( this, connName );
