@@ -61,6 +61,11 @@ QString QgsTextBlock::toPlainText() const
   return res;
 }
 
+void QgsTextBlock::reserve( int count )
+{
+  mFragments.reserve( count );
+}
+
 void QgsTextBlock::append( const QgsTextFragment &fragment )
 {
   mFragments.append( fragment );
@@ -71,6 +76,16 @@ void QgsTextBlock::append( QgsTextFragment &&fragment )
   mFragments.push_back( fragment );
 }
 
+void QgsTextBlock::insert( int index, const QgsTextFragment &fragment )
+{
+  mFragments.insert( index, fragment );
+}
+
+void QgsTextBlock::insert( int index, QgsTextFragment &&fragment )
+{
+  mFragments.insert( index, fragment );
+}
+
 void QgsTextBlock::clear()
 {
   mFragments.clear();
@@ -86,6 +101,11 @@ int QgsTextBlock::size() const
   return mFragments.size();
 }
 
+void QgsTextBlock::setBlockFormat( const QgsTextBlockFormat &format )
+{
+  mBlockFormat = format;
+}
+
 void QgsTextBlock::applyCapitalization( Qgis::Capitalization capitalization )
 {
   for ( QgsTextFragment &fragment : mFragments )
@@ -94,6 +114,12 @@ void QgsTextBlock::applyCapitalization( Qgis::Capitalization capitalization )
   }
 }
 
+bool QgsTextBlock::hasBackgrounds() const
+{
+  return mBlockFormat.hasBackground()
+  || std::any_of( mFragments.begin(), mFragments.end(), []( const QgsTextFragment & fragment ) { return fragment.characterFormat().hasBackground(); } );
+}
+
 const QgsTextFragment &QgsTextBlock::at( int index ) const
 {
   return mFragments.at( index );
